function [] = Animate( solcon, color, tpause )
% ANIMATE animates the five-link SemiQuad robot given a concatenated
% solution of the hybrid reduced dynamics
%
%[] = Animate( solcon , figurehandle1, figurehandle2, color, tpause, surface )
% solcon ... concatenated solution of a simulation of the hybrid reduced
%            dynamics
% figurehandle1, figurehandle2 ... handles to figures. figurehandle1 =
%                                  stick figure, figurehandle2 = phase plot
% color ... line color of the plots
% tpause ... time between 2 animation steps

figurehandle2 = figure(2); hold on;
figurehandle1 = figure(1); hold on;

if (nargin < 4)
    color = 'b';
end
if (nargin<5)
    tpause=0.05;
end

surface = [];

x = solcon.xp;
y = solcon.yp;
th = solcon.y(:,1);
dth = solcon.y(:,2);

stop = false;

%% Plotting routine

%set (SurfaceModel, 'xdata', surface(:,1), 'ydata', surface(:,2));
h2 = figure(figurehandle2);
PhaseModel = line(0,0,'color',color,'EraseMode','none','linewidth',1);
PhaseTitle = title('Phase plot');
xlabel('q_1');
ylabel('dq_1');
axis([min(th(:)) max(th(:)) min(dth(:)) max(dth(:))]);

h1 = figure(figurehandle1);
xlabel('x (m)');
ylabel('y (m)');
grid on
HandModel = line(0,0,'color',color,'Marker','.');
SurfaceModel = line(0,0,'color','b','linewidth',1);
HandTitle = title('Motion Trajectories') ;
axis equal;
axis([-1.0 1.0 -1 1]);

% xor for update behaviour
set (HandModel,'EraseMode','none');
set (HandTitle,'EraseMode','none');

%% Stop Button
stop = false;
pos = get(0,'DefaultUicontrolPosition');
pos(1) = pos(1) - 15;
pos(2) = pos(2) - 15;
uicontrol( ...
    'Style','togglebutton', ...
    'String','Stop', ...
    'Position',pos, ...
    'Callback',@StopButtonCallback, ...
    'Tag','stop');


%% Animation
grid on;
%axis equal;

animlength = length(x);
for i=1:animlength
    co = [ x(i,:).' y(i,:).'];

    set(HandModel,'xdata',co(:,1),'ydata',co(:,2));
    titlestring = ['Time Step ' num2str(i) '/' num2str(animlength)];
    set(HandTitle,'String',titlestring);
    if ~isempty(th)
        set(PhaseModel,'xdata', th(1:i), 'ydata', dth(1:i));
    end
    drawnow;
    if stop
        while stop
            pause(tpause);
        end
    else
        pause(tpause);
    end
end

%axis tight;


%% Stop button callback function
    function StopButtonCallback(src,eventdata)
        stop = ~stop;
    end 

end