function [lin_sol] = CartLinearDynamicsEquation(tspan,G_funcs,sol,catchevent)

g_y = G_funcs{1};
g_dy = G_funcs{2};
g_ddy = G_funcs{3};
Alpha = G_funcs{4};
Beta = G_funcs{5};
Gamma = G_funcs{6};
L = 0.15;
g = 9.81;
th = sol.y(:,1);
dth = sol.y(:,2);
I0 = 0.005;
y0 = 0;
dy0 = 0;
ct = 0;
n = length(sol.y(:,1))

%Calculating the Constant Gain Matrix K
% ddth = -(Beta(th(ct),L)*dth(ct)^2+Gamma(th(ct),L,g))/Alpha(th(ct),L);
%         
% a_11 = (-2*dth(ct)/Alpha(th(ct),L))*Beta(th(ct),L);
% a_12 = (2*dth(ct)/Alpha(th(ct),L))*g_y(th(ct),ddth,L,g);
% a_13 = (2*dth(ct)/Alpha(th(ct),L))*g_dy(th(ct),ddth,L,g);
% b_1 = 2*dth(ct)/Alpha(th(ct),L)*g_ddy(th(ct),ddth,L,g);
% 
% A0 = [a_11 a_12  a_13;
%      0      0       1;
%      0      0       0];
%   
% B0 = [b_1; 0; 1];
% 
Q = 100*eye(3);
R = 0.1;
S = zeros(3,1);
E = eye(3);
% 
% %Solving the Riccati Equation
% [X,L_m,G] = care(A0,B0,Q,R,S,E);

REFINE = 4;
 
options = odeset('Events',@Event,'Refine',REFINE);
% doc odeset:
% Refine | Positive integer | Increases the number of output points by a factor of Refine.
% Events | Function handle  | Handle to a function that includes one or more
%                             event functions.

[ lin_sol.t,lin_sol.y,lin_sol.te,lin_sol.ye,lin_sol.ie ] = ode45(@DynamicsEquations, tspan, [I0 y0 dy0], options);
% doc ode45:
% Corresponding entries in TE, YE,
% and IE return, respectively, the time at which an event
% occurs, the solution at the time of the event, and the index i of
% the event function that vanishes.
   

 function [dx] = DynamicsEquations(t, x)
        ct = ct+1;
        I_temp = x(1);
        y_temp = x(2);
        dy_temp = x(3);
        ddth = -(Beta(th(ct),L)*dth(ct)^2+Gamma(th(ct),L,g))/Alpha(th(ct),L);
        
        a_11 = (-2*dth(ct)/Alpha(th(ct),L))*Beta(th(ct),L);
        a_12 = (2*dth(ct)/Alpha(th(ct),L))*g_y(th(ct),ddth,L,g);
        a_13 = (2*dth(ct)/Alpha(th(ct),L))*g_dy(th(ct),ddth,L,g);
        b_1 = 2*dth(ct)/Alpha(th(ct),L)*g_ddy(th(ct),ddth,L,g);
        
        A = [a_11 a_12  a_13;
             0      0       1;
             0      0       0];
        B = [b_1; 0; 1];
        
        %Include The Riccati Calculated Update Law for the Control
        [X,L_m,G] = care(A,B,Q,R,S,E);
        eig_norm=eig(A);
        eig_control=eig(A-B*G);
        x
        %G = zeros(1,3);
        dx = (A - B*G)*x    %3x1 equation, what is v??? v is control
end

    function [value,isterminal,direction] = Event(t,x)
            value = 0;
            isterminal = 1;
            direction = 0;
    end
end