function [solExt] = ExtendedSolution(sol,Dyn_Syst)
%Extends the solution to also include the position (x,y) of all joints and
%tip of swing leg in addition to extending to include a whole gait, or 
%several gaits. Useful for extra research and for plotting.
%function [solExt] = ExtendedSolution(sol)
%Input...
%sol...     A struct containing values of the polynomial parameters in
%addtion to the values of theta and theta derivative for all time values
%Output...
%SolExt...  Extended solution containing also the positon (x,y) of all
%joints and tip of swing leg.


solExt.xp = [];
solExt.yp = [];
solExt.t = [];
solExt.y = [];
solExt.te = [];
solExt.ye = []; 
solExt.q = [];
solExt.error = [];
solExt.torque = [];

for i = 1:length(sol)
    if i == 1
        solExt.t = sol(i).t;
        solExt.te = sol(i).te;
    else
        solExt.te = [solExt.te; solExt.t(end) + sol(i).te];
        solExt.t = [solExt.t; solExt.t(end) + sol(i).t];
    end
    
    solExt.y = [solExt.y; sol(i).y];
    solExt.q = [solExt.q; sol(i).y(:,1) theta2phi(sol(i).y(:,1), sol(i).CP)];
    solExt.error = [solExt.error; sol(i).error];
    %solout.extForce = [solout.extForce; solin(ii).extForce];
    
    solExt.torque = [solExt.torque; sol(i).torque];
  
end
 
%Fixing the positions
sessions = length(sol);

PCM = Dyn_Syst{3};

offset{1} = 0;

for i=1:sessions,
    q = [sol(i).y(:,1) theta2phi(sol(i).y(:,1), sol(i).CP)];
   
    [xp{i},yp{i}] = ForwardKinematics(q,PCM);
    
    xp{i} = [zeros(length(xp{i}),1) xp{i}];
    yp{i} = [zeros(length(yp{i}),1) yp{i}];
    
    if mod(i,4) == 3 || (mod(i,4) == 1 && i > 3)
         offset{i+1} = offset{i} + xp{i-1}(end,6);
    else
         offset{i+1} = offset{i};
    end
    solExt.xp = [solExt.xp; xp{i}+offset{i+1}];
    solExt.yp = [solExt.yp; yp{i}];
end

end

    
    
    
     
        