function [A] = matrix_A_timeFunction(sol,solExt,Dyn_Syst,LinSys,t,ite)
%Function to give the value of matrix A from the linear dynamics given by 
%dx = Ax + Bv.
%Input... 
%sol... solution of nonlinear System
%solExt... extended nonlinear solution 
%Dyn_Syst... function handles to nonlinear system
%Linsys... function handles to linear system
%t... specific time t
%ite... current iteration.

%Function Handles and parameters
g_y = LinSys{1};
g_dy = LinSys{2};
g_ddy = LinSys{3};

abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

[l, s, m, I_m, I_r, N, g] = RobotParameters();

PQ = sol(ite).CP;

%Extracting theta-dtheta and converting to time functions
the = thetaTimeFunction(sol,solExt,t,ite);
dthe = dthetaTimeFunction(sol,solExt,t,ite);

%calculating ddthe
ddthe = -(Beta(l,m,s,I_r,PQ,the)*dthe^2+Gamma(l,m,s,g,PQ,the))/Alpha(l,m,s,I_r,PQ,the);

gain = (2*dthe)/(Alpha(l,m,s,I_r,PQ,the));

a_11 = -gain*Beta(l,m,s,I_r,PQ,the);
a_12 = gain*g_y(PQ,l,m,s,the,g,dthe,ddthe)*(-1);
a_13 = gain*g_dy(PQ,l,m,s,the,dthe)*(-1);

A =  [a_11          a_12            a_13;
      zeros(4,1)    zeros(4,4)      eye(4);
      zeros(4,1)    zeros(4,4)      zeros(4,4)];

end

  
  
% %Extracting theta-dtheta and converting to time functions
% the = @(t)  thetaTimeFunction(sol,solExt,t,ite);
% dthe = @(t) dthetaTimeFunction(sol,solExt,t,ite);
% 
% %Changing Function Handles to only be dependen on variable t
% Alpha = @(t) Alpha_full(l,m,s,I_r,PQ,the(t)); 
% Beta = @(t) Beta_full(l,m,s,I_r,PQ,the(t));  
% Gamma = @(t) gamma_full( l,m,s,g,PQ,the(t)); 
%  
% %calculating ddthe
% ddthe = @(t) -(Beta(t)*dthe(t)^2+Gamma(t))/Alpha(t);
% 
% gain = @(t) (2*dthe(t))/(Alpha(t));

% %Function Handles and parameters
% g_y_full = LinSys{1};
% g_dy_full = LinSys{2};
% g_ddy_full = LinSys{3};
% 
% abg = Dyn_Syst{1};
% Alpha_full = abg{1};
% Beta_full = abg{2};
% Gamma_full = abg{3};
