function [B] = matrix_B_timeFunction(sol,solExt,Dyn_Syst,LinSys,t,ite)
%Function to give the value of matrix A from the linear dynamics given by 
%dx = Ax + Bv.
%Input... 
%sol... solution of nonlinear System
%solExt... extended nonlinear solution 
%Dyn_Syst... function handles to nonlinear system
%Linsys... function handles to linear system
%t... specific time t
%ite... current iteration.

%Function Handles and parameters
g_y = LinSys{1};
g_dy = LinSys{2};
g_ddy = LinSys{3};

abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

[l, s, m, I_m, I_r, N, g] = RobotParameters();

PQ = sol(ite).CP;

%Extracting theta-dtheta and converting to time functions
the = thetaTimeFunction(sol,solExt,t,ite);
dthe = dthetaTimeFunction(sol,solExt,t,ite);

%calculating ddthe
ddthe = -(Beta(l,m,s,I_r,PQ,the)*dthe^2+Gamma(l,m,s,g,PQ,the))/Alpha(l,m,s,I_r,PQ,the);

gain = (2*dthe)/(Alpha(l,m,s,I_r,PQ,the));

b_1 = gain*g_ddy(PQ,l,m,s,the,I_r)*(-1);

B = [b_1;zeros(4,4); eye(4)];

end
