function [Linsol] = simulateLinearSystem_premade(I0_y0_dy0,A,B,T_start,ite)
%Function that simulates a single step of the linear system using the
%precalculated A and B matrices.

tspan = [0 3];

I0 = I0_y0_dy0(1);
y0 = I0_y0_dy0(2:5);
dy0 = I0_y0_dy0(6:9);

t_tot = T_start(ite+1)-T_start(ite);

%Solving the First ODE

REFINE = 4;

options = odeset('Events',@Event,'Refine',REFINE,'RelTol',1e-6,'AbsTol',1e-9);
% doc odeset:
% Refine | Positive integer | Increases the number of output points by a factor of Refine.
% Events | Function handle  | Handle to a function that includes one or more
%                             event functions.

[ Linsol.t,Linsol.y,Linsol.te,Linsol.ye,Linsol.ie ] = ode45(@DynamicsEquations, tspan, [I0 y0(1) y0(2) y0(3) y0(4) dy0(1) dy0(2) dy0(3) dy0(4)], options);
% doc ode45:
% Corresponding entries in TE, YE,
% and IE return, respectively, the time at which an event
% occurs, the solution at the time of the event, and the index i of
% the event function that vanishes.

function [dx] = DynamicsEquations(t,x)
     
    t_ad = t + T_start(ite); 
    v = zeros(4,1);
  
    dx = A(t_ad)*x+B(t_ad)*v;
end

function [value,isterminal,direction] = Event(t,x)
        value = t-t_tot;
        isterminal =1;
        direction = 0;
end
end