function [L] = switch_Matrix_L(sol,solExt,Dyn_Syst,ite)
%Function to find the corresponding update matrix L such that 
% X_trans_new = L_ite * X_trans_old for the given switch (1-4). Switch 1 is
% then the switch between step 1 and step 2 and step 4will be the switch
% between step 4 and step 1.
%Input...
%sol... A stored solution for the nonlinear system.
%Dyn_Syst... function handles for the nonlinear system.
%ite... the current iteration (or switch) you want to calculate.

%Extracting function handles and system parameters
abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

[l, s, m, I_m, I_r, N, g] = RobotParameters();

%Extracting time of Switch
t = solExt.te(ite);

PQ_e = sol(ite).CP;
if ite == 4
    ite_pos = 1;
    PQ_0 = sol(ite_pos).CP;
    t_pos = 0;
else,
    ite_pos = ite+1;
    PQ_0 = sol(ite_pos).CP;
    t_pos = t;
end

%Extracting proper values of theta and dtheta before switch
the_e = thetaTimeFunction(sol,solExt,t,ite);
dthe_e = dthetaTimeFunction(sol,solExt,t,ite);

%Extracting optimal values of ddthe, q, dq, ddq before switch
ddthe_e = -(Beta(l,m,s,I_r,PQ_e,the_e)*dthe_e^2+Gamma(l,m,s,g,PQ_e,the_e))/Alpha(l,m,s,I_r,PQ_e,the_e);

[q_e,dq_e,ddq_e,dPhi_e,ddPhi_e] = extractGeneralizedVariables(the_e,dthe_e,ddthe_e,PQ_e);

%Extracting proper values of theta and dtheta before after switch
the_0 = thetaTimeFunction(sol,solExt,t_pos,ite_pos);
dthe_0 = dthetaTimeFunction(sol,solExt,t_pos,ite_pos);

%Extracting optimal values of ddthe, q, dq, ddq after switch
ddthe_0 = -(Beta(l,m,s,I_r,PQ_0,the_0)*dthe_0^2+Gamma(l,m,s,g,PQ_0,the_0))/Alpha(l,m,s,I_r,PQ_0,the_0);

[q_0,dq_0,ddq_0,dPhi_0,ddPhi_0] = extractGeneralizedVariables(the_0,dthe_0,ddthe_0,PQ_0);
 
%Extracting L_e and N_e
[L_e,N_e] = extract_L_N(dthe_e,ddthe_e,dq_e,ddq_e,dPhi_e,ddPhi_e);

%Extracting L_0 and N_0
[L_0,N_0] = extract_L_N(dthe_0,ddthe_0,dq_0,ddq_0,dPhi_0,ddPhi_0);

L_N_e = [L_e;N_e];
L_N_0 = [L_0;N_0];

%Extracting the m vector orthogonal to switching surface
if ite == 1 || ite == 3,
    m = [0,0,0,0,0,1,0,0,0,0];
else,
    m = [l(1)*cos(the_e)+l(2)*cos(the_e+q_e(1))+l(3)*cos(the_e+q_e(1)+q_e(2))+...
         +l(4)*cos(the_e+q_e(1)+q_e(2)+q_e(3))+l(5)*cos(the_e+q_e(1)+q_e(2)+q_e(3)+q_e(4));
         l(2)*cos(the_e+q_e(1))+l(3)*cos(the_e+q_e(1)+q_e(2))+...
         +l(4)*cos(the_e+q_e(1)+q_e(2)+q_e(3))+l(5)*cos(the_e+q_e(1)+q_e(2)+q_e(3)+q_e(4));
         l(3)*cos(the_e+q_e(1)+q_e(2))+...
         +l(4)*cos(the_e+q_e(1)+q_e(2)+q_e(3))+l(5)*cos(the_e+q_e(1)+q_e(2)+q_e(3)+q_e(4));
         l(4)*cos(the_e+q_e(1)+q_e(2)+q_e(3))+l(5)*cos(the_e+q_e(1)+q_e(2)+q_e(3)+q_e(4));
         l(5)*cos(the_e+q_e(1)+q_e(2)+q_e(3)+q_e(4));0;0;0;0;0]';
end

%Constants
K1 = m*N_e';
K2 = N_0*N_0';
L_N_inv = inv(L_N_e);

if ite == 2 || ite ==4,
PQ = sol(ite).CP;
[U1, U2] = getImpactModel(Dyn_Syst,sol,PQ,ite);
Imp = U1\U2   
% F = [1     1     1     1     1     0     0     0     0     0;
%      0     0     0     0    -1     0     0     0     0     0;
%      0     0     0    -1     0     0     0     0     0     0;
%      0     0    -1     0     0     0     0     0     0     0;
%      0    -1     0     0     0     0     0     0     0     0;
%      0     0     0     0     0     1     1     1     1     1;
%      0     0     0     0     0     0     0     0     0    -1;
%      0     0     0     0     0     0     0     0    -1     0;
%      0     0     0     0     0     0     0    -1     0     0;
%      0     0     0     0     0     0    -1     0     0     0];
F_1 = [1     1     1     1     1      
       0     0     0     0    -1      
       0     0     0    -1     0     
       0     0    -1     0     0     
       0    -1     0     0     0];
   
F = [F_1 zeros(5,5); zeros(5,5) F_1*Imp]
else,
    F = eye(10);
end

%The new L
% L_1 = F*(-1/K2)*((-1/K1)*(N_0'*N_0*N_e'*m*L_N_inv)+N_0'*N_0*L_N_inv);
% L_2 = F*(-1/K1)*(N_e'*m*L_N_inv);
% L_3 = F*L_N_inv;
% 
% N_0
% N_e
% m 
% 
% 
% L_N_inv 

L_1 = (-N_0'*N_0/K2 + eye(10))*F*((-N_e'*m*L_N_inv)*(1/K1) + L_N_inv);

% W_4 = (-N2*N2'/K2 + eye(2))*F*(-N1*m'*L_n_inv/K1 + L_n_inv)*fy 

Ip = [eye(9);zeros(1,9)];

% L = L_0*(L_1+L_2+L_3)*Ip;
L = L_0*L_1*Ip;

% x_t = ones(10,1);
% 
% L_c_1 = F*L_1
% 
% L_c_2 = L_N_inv*x_t*m
% 
% L_c_3 = (m*L_N_inv*x_t)
% 
% 
% 
% L = 1;

end
















