function [P_end,dtheta_opt,thetae_opt, stat] = OptimizedGait(theta0,dtheta0,thetae_0,Pos,Dyn_Syst)
%Optimized a given gait (starting point) with respect to specific
%contraints and specific cost functions. 
%Input...
%theta0... starting point for ankle
%dtheta0... inital angular velocity of ankle
%pos... starting configuration for the reference positions of the various
%steps.
%Dyn_Syst... Cell contaning the function handles of the various dynamic
%system expressions and matrices.
%Output... the optimized values of:
%P_end... reference positions
%dtheta_opt... starting initial angular velocity
%thetae_opt... values for reference ending value of theta to be used in
%steps 1 and 3 of the gait. Changing these values can make the polynomial
%of the VHC perform better without actually reaching these ending value.

global NUM_GAITS
global MAX_FUN_EVALS
global NUM_VHC_PARAM
global OPTIM_ALG
global TMAX
global Step_Length

%% Definition of constants 

tspan = [0 TMAX];

%% Cost function definition

    function [f,g,H ] = fun(x)
        P = updatePos(x,Pos,Step_Length,NUM_VHC_PARAM-1);
        dtheta_opt = x(end);
        thetae_opt = x(end-2:end-1);
        %TO BE REMOVED
%         P = Pos;
%         P_end(:,2,1) = x(1:4);
%         P_end(:,3,1) = x(5:8);
%         P_end(:,4,1) = x(9:12);
%         P_end(:,3,2) = x(1:4);
%           P_end(2,3,2) = x(1);
%           P(:,3,4) = x(1:4);
%           thetae_opt = thetae_0;
%           dtheta_opt = dtheta0;
        x
        [f] = optimizationCostFunction(tspan, theta0, dtheta_opt,thetae_opt, P, Dyn_Syst);
        f
    end

x0  = getOptimizableParameters(Pos,dtheta0,thetae_0,Step_Length,NUM_VHC_PARAM-1)               
% x0 = [Pos(:,3,3)' Pos(:,4,3)' Pos(:,3,4)' thetae_0 dtheta0]
%To be removed!!
% x0 = [x0_0(25:28)] 
% x0 = [ -1.5823   -0.8518   -0.8296   -1.4878   -1.6046   -0.9707   -0.7461   -1.5698   -1.3323   -0.9569 -0.4615   -1.5129   -1.5330   -0.8320   -0.6921   -1.5102    3.7869    0.3317    4.9041] 
switch lower(OPTIM_ALG) 
    
    case {'fminsearch'}
        options =  optimset('Display','notify','TolFun',1e-1,'MaxFunEvals',MAX_FUN_EVALS,'LargeScale','on');
        [opt_param,fval,exitflag,output] = fminsearchBigStep(@fun,x0,options);
        stat = struct('fval',fval);
    case {'simulated annealing'}
        %options = struct('InitTemp',1,'StopTemp',1e-8,'StopVal',-Inf,'CoolSched',@(T)(.8*T),'MaxIter',MAX_FUN_EVALS);
        options = struct('MaxIter',MAX_FUN_EVALS);
        [opt_param,stat] = anneal(@fun,x0,options);
        %fval = stat.fval;
        %figure; plot(stat.fvals);
    otherwise
        disp('OPTIM_ALG value invalid');
end

dtheta_opt = opt_param(end);
thetae_opt = opt_param(end-2:end-1);
% P = Pos;
% P(:,3,4) = opt_param(1:4);
% x0_0  = getOptimizableParameters(P,dtheta0,thetae_0,Step_Length,NUM_VHC_PARAM-1);
% x0_0(25:28) = opt_param;
P_end = totalPos(opt_param,Step_Length,NUM_VHC_PARAM-1,thetae_opt,theta0,dtheta_opt,tspan,Dyn_Syst);
end

 