function [ dx ] = ReducedDynamics( x , PQ, abg)
%REDUCEDDYNAMICS calculates dx = [ d/dt(theta); d^2/dt^2(theta)]
%according to the alpha-beta-gamma equation of the reduced dynamics
%
%[ dx ] = ReducedDynamics( x , PQ)
% x ... state vector (= [ theta ; d/dt(theta) ])
% PQ ... parameters of the 2nd order polynomial virtual holonomic
%        constraints
%returns:
% dx ... d/dt(x)


th = x(1);
dth = x(2);

[ Alpha, Beta, Gamma ] = AlphaBetaGamma( th, PQ, abg );

ddth = -(Beta*dth^2+Gamma)/Alpha;

dx = [dth; ddth];

end
