function [PQ] = SingleOptimizedGait(theta0,thetae,dtheta0,Pos,Dyn_Syst)
%Used to estimate a single step of the gait, in order to find valid
%starting configurations of the VHC parameters.
%Input...
%Theta0... starting angle for the ankle
%dtheta0... starting angular velocity for the ankle
%phi0 starting configuration for the joints
%phie... Ending configuration for the joints

global MAX_FUN_EVALS
global NUM_VHC_PARAM
global OPTIM_ALG
global TMAX

phie = Pos(:,3);

%% Definition of constants 

tspan = [0 TMAX];
 
%% Cost function definition

    function [f,g,H ] = fun(x)
        %P = [x(1:4)' x(5:8)' x(9:12)'];
        P = [Pos(:,1) x' Pos(:,3)];
        x
        PQ = bezier2dim(P,theta0,thetae);
        f = SingleOptimizationCostFunction(tspan, theta0, thetae, dtheta0, PQ,Dyn_Syst, phie);
        f
        % Compute the objective function value at x
        % if nargout > 1   % fun called with two output arguments
        %    g = ...  % Gradient of the function evaluated at x
        %    if nargout > 2
        %       H = ...  % Hessian evaluated at x
        %    end
        % end
    end


%% Optimize

%Optimizing with respect to the midpoint values

%x0 = [Pos(:,1); Pos(:,2); Pos(:,3)]; 
x0 = Pos(:,2);

switch lower(OPTIM_ALG) 
    
    case {'fminsearch'}
        options =  optimset('Display','notify','TolFun',1e-1,'MaxFunEvals',MAX_FUN_EVALS,'LargeScale','on');
        [opt_param,fval,exitflag,output] = fminsearch(@fun,x0(:)',options);
        stat = struct('fval',fval);
    case {'simulated annealing'}
        %options = struct('InitTemp',1,'StopTemp',1e-8,'StopVal',-Inf,'CoolSched',@(T)(.8*T),'MaxIter',MAX_FUN_EVALS);
        options = struct('MaxIter',MAX_FUN_EVALS);
        [opt_param,stat] = anneal(@fun,x0(:)',options);
        %fval = stat.fval;
        %figure; plot(stat.fvals);
    otherwise
        disp('OPTIM_ALG value invalid');
end

P = [Pos(:,1) opt_param' Pos(:,3)];
PQ = bezier2dim(P,theta0, thetae);

end