function [Tau] = Torque(sol, PQ, Dyn_Syst)
%Calculates the Torque in each actuated joint.
%Input...
%Sol, current solution of the reduced system dynamics
%PQ, coefficients of the virtual holonomic contrains
%abg, function handle to the alpha,beta, gamma functions
%tor, function handle to the torque function.

[l, s, m, I_m, I_r, N, g] = RobotParameters();

th = sol.y(:,1);
% dth = sol.y(:,2);
dth = sol.y(:,6);
abg = Dyn_Syst{1};
tor = Dyn_Syst{2};

step = length(th);
Tau = zeros(step,4);

for i=1:step,
    [alpha, beta, gamma] = AlphaBetaGamma(th(i),PQ,abg);
    ddth = -(beta*dth(i)^2+gamma)/alpha;
    torque = tor(l,m,s,g,I_r,I_m,N,PQ,th(i),dth(i),ddth)'
    Tau(i,:) = torque;
end

