function [q,dq,ddq,ddth] = getGeneralizedCoordinates(sol,PQ_tot,Dyn_Syst,ite)
%Function to extract the generalized coordinates q together with their
%first and second derivatives. They are then returned as a an array
%calculated for all values of th in the optimized trajectory.
%Input... 
%sol... the calculated solution for a certain type of step,
%containing the time evolution of th.
%PQ... the optimized coefficients of the parametrization of generalized
%coordinates expressed as a function of th
%Dyn_Syst... the function handles to the dynamic system

% Extracting Function 
abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

[l, s, m, I_m, I_r, N, g] = RobotParameters();

%Calculating the coefficients 

th = sol(ite).y(:,1);
dth = sol(ite).y(:,2);

dim = length(PQ_tot(1,:,1));
n = length(th);
q = zeros(n,4);
dq = zeros(n,4);
ddq = zeros(n,4);
PQ = PQ_tot(:,:,ite);
ddth = zeros(n,1);


for i = 1:n,
    ddth(i) = -(Beta(l,m,s,I_r,PQ,th(i))*dth(i)^2+Gamma(l,m,s,g,PQ,th(i)))/Alpha(l,m,s,I_r,PQ,th(i));
    for j = 1:4,
        for k = 1:dim,
            q(i,j) = PQ(j,k)*th(i)^(dim-k) + q(i,j);
            dq(i,j) = (dim-k)*PQ(j,k)*th(i)^(dim-k-1) + dq(i,j);
            ddq(i,j) = (dim-k)*(dim-k-1)*PQ(j,k)*th(i)^(dim-k-2) +ddq(i,j);
        end
        ddq(i,j) = ddq(i,j)*dth(i)^2 + ddth(i)*dq(i,j);
        dq(i,j) = dq(i,j)*dth(i);
    end
end

end

