function [L,N] = getLinear2NonMatrices(th,dth,ddth,dq,ddq,PQ,ts)
%Function designed to calculate the matrices L and N needed to convert the
%nonlinear parameters into transvere parameters and vica versa.
%Input... 
%th... true value of theta from the solution of the nonlinear solution.
%Likewise for dth, ddth, dq and ddq.
%PQ... the curent parametrization coefficients for the current iteration of
%the system.
%Step... The current solution step that should be solved.
%Output...
%L... a matrix making up the main bulk of the transformation between the
%original parameters and the transvere parameters. 
%N... The vector normal to the solution plane of the system being the
%individual derivatives 

dim = length(PQ(1,:));

%Creating Phi Expressions
dphi = zeros(1,4);
ddphi = zeros(1,4);

for j=1:4,
    for k=1:dim-1,
        dphi(j)= dphi(j) + (dim-k)*PQ(j,k)*th(ts)^(dim-k-1);
        ddphi(j) = ddphi(j) + (dim-k)*(dim-k-1)*PQ(j,k)*th(ts)^(dim-k-2);
    end
end

%Creating the L matrix (from #5)

L = [-2*ddth(ts) zeros(1,4) 2*dth(ts) zeros(1,4); 
     -dphi(1)  1 zeros(1,8);
     -dphi(2)  0 1 zeros(1,7);
     -dphi(3)  0 0 1 zeros(1,6);
     -dphi(4)  0 0 0 1 zeros(1,5);
     -ddphi(1)*dth(ts) zeros(1,4) -dphi(1) 1 zeros(1,3);
     -ddphi(2)*dth(ts) zeros(1,4) -dphi(2) 0 1 zeros(1,2); 
     -ddphi(3)*dth(ts) zeros(1,4) -dphi(3) zeros(1,2) 1 0;
     -ddphi(4)*dth(ts) zeros(1,4) -dphi(4) zeros(1,3) 1;
     ];

%Creating the n-matrix (from #5)
N = [dth(ts) dq(ts,:) ddth(ts) ddq(ts,:)];