function [A, B] = getLinearMatrices(the,dthe,Dyn_Syst,LinSys)
%Function to generate the two matrices A and B to write the linear system
%dx = Ax + Bv with the transverse coordinates x.
%Input...
%the... current optimal value of coefficient theta wrt time
%dthe... current optimal value of coefficient dtheta wrt time
%Dyn_Syst... collection of function handles for the dynamic system 
%LinSys... collection of function handles for the transverse linearization

%Function Handles and parameters
g_y = LinSys{1};
g_dy = LinSys{2};
g_ddy = LinSys{3};

abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

[l, s, m, I_m, I_r, N, g] = RobotParameters();

%The coefficients 

ddthe = -(Beta(l,m,s,I_r,PQ,the)*dthe^2+Gamma(l,m,s,g,PQ,the))/alpha;

a_11 = -gain*Beta(l,m,s,I_r,PQ,the);
a_12 = gain*g_y(PQ,l,m,s,the,g,dthe,ddthe)*(-1);
a_13 = gain*g_dy(PQ,l,m,s,the,dthe)*(-1);
b_1 = gain*g_ddy(PQ,l,m,s,the,I_r)*(-1);

 A = [a_11          a_12            a_13;
      zeros(4,1)    zeros(4,4)      eye(4);
      zeros(4,1)    zeros(4,4)      zeros(4,4)];

 B = [b_1;zeros(4,4); eye(4)];
 
 end