function [PQ] = getSolutionCoeff(Pos_e,sol,thetae_opt,Dyn_Syst)
%A function designed to return the coefficients that were used to discover
%a specific solution for the gait.
%Input...
%Pos_e... the ending positional matrices 
%sol... the solution of each step with given values
%theta_opt... the discovered optimal ending values of theta
%Dyn_Syst... the function handles for the dynamic system
%Output... 
%PQ... the coefficients for the parametrization of the DOF

dim = length(Pos_e(1,:,1));

PQ = zeros(4,dim,4);

for i=1:4,
% for i=1:3,
    P = Pos_e(:,:,i);
    theta0 = sol(i).y(1,1);
    if even(i),
        thetae=sol(i).y(end,1);
    else
        if i==1,
            thetae=thetae_opt(1);
        elseif i==3, 
            thetae=thetae_opt(2);
        end
    end
    PQ(:,:,i) = Bezier(P,theta0,thetae,Dyn_Syst);
end

end

    
    
    

