function [ retval ] = isAboveFloor( theta, PQ, PCM)
%ISABOVEFLOOR returns if all the joints of the robot are above the floor or
% not. 
%
%[ retval ] = isAboveFloor( theta, PQ )
% theta ... measure of progress, scalar or vector of values over time
% PQ ... coefficients of polynomial virtual holonomic constraints
%returns:
% retval ... true if the robot is above the floor for all values of theta,
%            else false

global EPSILON;

retval = false;

[x,y] = ForwardKinematics([theta theta2phi(theta,PQ)], PCM);

if isempty(find(y < -EPSILON, 1))
    retval = true;
end
