function [ violated, torque, count] = mechanicalconstraints( sol, PQ, tspan, Dyn_Syst, ite)
%MECHANICALCONSTRAINTS checks if SemiQuad is above the floor, if the
% switching event occured (if it didn't occur, the motion is considered too
% slow or not periodic) and if the maximum torque was exceeded.
%
%[ violated, torque ] = mechanicalconstraints( sol, PQ, tspan )
% sol ... is a solution of the (hybrid) reduced dynamics
% PQ ... is the set of virtual holonomic constraint parameters
% tspan ... time span of sol
%returns:
% violated ... true if the constraints are violated, else false
% torque ... matrix containing the torque values of the 4 actuated joints
%            over time

global MAX_TORQUE %maximal torque allowed [Nm]
Disp_Out = num2str(ite);

violated = true;
torque = Inf;
count = zeros(3,1);

PCM = Dyn_Syst{3};

% check if maximum torque was exceeded at any time
torque = Torque(sol,PQ,Dyn_Syst);
if ~isempty(find(abs(torque) > MAX_TORQUE, 1))
    sprintf('Torque limit Breached in step %d',ite)
    count(1) = 1;
end

%check if robot is above floor at all time values
if ~isAboveFloor(sol.y(:,1), PQ, PCM)
    sprintf('Floor limit Breached in step %d',ite)
    count(2) = 1;
end



% check if switch event occured
if sol.t(end) == tspan(end) || length(sol.ie) < 1 || (sum(isnan(sol.y(:))) > 0)
    sprintf('Timer exceeded in step %d',ite)
    count(3) = 1;
    return
end

if sum(count) > 0,
    return
end

violated = false;

end