function [ f ] = optimizationCostFunction( tspan, theta0, dtheta0,thetae_opt, P, Dyn_Syst)
%[ f,g,H ] = optimizationCostFunction( tspan, theta0, phi0, dtheta0, P )
%Calculates a cost function based either on the sciencemag approach or on 
%kinetic energy of a gait of the semiquad, the purpose naturally being to 
%limit these. In addition several other expressions are included in the 
%cost function to ensure a functioning gait. Temp, is a variable that becomes
%active if there are errors, introducing a heavy punishment to the cost 
%function. Dtheta0 = Dthetae ensures that the end configuration of the 
%angular velocity (after impact and switch) is the same as the starting
%angular velocity, so our gait will be in the exact same configuration and 
%therefore be periodic.
%Input...
%tspan... longest allowable time for a single step of the gait, exceeding 
%this results in an error.
%theta0... starting value of theta
%dtheta0... current optimal value for the initial angular velocity
%thetae_opt... current optimal values for the thetae in step 1 and 3
%P... current optimal configuration of optimal positonal value + the 
%mandatory preconfigured values.
%Dyn_Syst... Cell containing the function handles of the dynamic_system
%Ouput...
%f... Value of the costfunction to be sent to the optimization routine
 
global OPTIMIZATION_CRITERIA
PCM = Dyn_Syst{3};

[solExt, sol, dtheta_end] = simulateAllSteps(tspan, theta0, dtheta0,thetae_opt, P, Dyn_Syst);

temp = 0;
% numError = sum(solExt.error);
% if numError ~= 0 
%     temp = numError*1e6;
%     % f = inf;
%      %return
% end

% temp = solExt.error(1)*1e25+solExt.error(2)*1e25;% +solExt.error(3)*1e6+solExt.error(4)*1e5;
% temp = temp + sol(3).count(1)*1e15+sol(3).count(2)*1e15;
% temp = temp + sol(4).count(1)*1e15+sol(4).count(2)*1e15;
% temp = temp + solExt.error(3)*1e35+solExt.error(4)*1e35;
temp = sol(1).count(1)*1e15+sol(2).count(1)*1e15+sol(3).count(1)*1e10+sol(4).count(1)*1e5;
temp = temp + sol(1).count(2)*1e35+sol(2).count(2)*1e35+sol(3).count(2)*1e25+sol(4).count(2)*1e20;
temp = temp + sol(1).count(3)*1e30+sol(2).count(3)*1e30+sol(3).count(3)*1e20+sol(4).count(3)*1e20;

ga = 1000;
ypsil = 1;% 10000;
EKin = zeros(4,1);
% dtheta_end

for ii = 1:4
% for ii = 1:3
    kk = length(sol)-4 + ii;
%     kk = length(sol)-3 + ii;
    PQ = sol(kk).CP;
    th = sol(kk).y(:,1);
    dth = sol(kk).y(:,2);
    q = [th theta2phi(th,PQ)];
    [xp,yp] = ForwardKinematics(q,PCM);
    torque = sol(kk).torque;
    dphi = dPhi(PQ,th,dth);
     energy_used(ii) = 0;
     for jj = 1:4
        energy_used(ii) = energy_used(ii) + abs(torque(jj,:))*abs(dphi(jj,:)'); % the second term is the energy introduced to the system
     end
end

[l, s, m, Ia, Iy, N, g] = RobotParameters();

t_tot = solExt.t(end);
valeris = 100;

dx = solExt.xp(1,end) - solExt.xp(1,1);
Energy_Costfunction = abs(sum(energy_used)/(sum(m)* dx))
dtheta_costFunction = ga*abs(dtheta0-dtheta_end)
Speed_CostFunction = valeris*t_tot;
% ga = 5;

%Bonus Optimization
% the_imp = sol(2).y(end,1);
% dthe_imp = sol(2).y(end,2)
% PQ_imp = sol(2).CP;
% dPhi_imp = getDPhi(PQ_imp,the_imp);
% dq_imp = dPhi_imp*dthe_imp
% 
% Imp_cost = abs(3.7617*dthe_imp+4.5347*dq_imp(1)+3.4304*dq_imp(2))
% Error_imp = sum(solExt.error(1:2));
% Temp_imp = 0;
% if Error_imp,
%     Temp_imp = 1e5;
% end
% 
% Imp_cost2 = abs(sol(3).y(1,2));
% 
ta = 1000;

%Optimization Attempt 2
% PQ = getSolutionCoeff(P,sol,thetae_opt,Dyn_Syst);
% PQ_3 = sol(3).CP;
% 
% theta0 = sol(3).y(1,1);
% dtheta0 = sol(3).y(1,2)
% 
% abg = Dyn_Syst{1};
% Alpha = abg{1};
% Beta = abg{2};
% Gamma = abg{3};
%  
% [l, s, m, I_m, I_r, N, g] = RobotParameters();
% 
% A = Alpha(l,m,s,I_r,PQ_3,theta0);
% B = Beta(l,m,s,I_r,PQ_3,theta0);
% G = Gamma(l,m,s,g,PQ_3,theta0);
% 
% 
% ddtheta0 = (-B*dtheta0^2-G)/A
% 
% Old_Cost = abs(ddtheta0-5.7836)+abs(dtheta0--2.5)

%Optimization attempt 3
% PQ = getSolutionCoeff(P,sol,thetae_opt,Dyn_Syst);
% PQ_3 = PQ(:,:,3)
% 
% load('Temp_PQ');
% PQ_3_nes = PQ_nes(:,:,3)
% 
% dim = length(PQ_3(1,:));
% len = 4;
% 
% dtheta0 = sol(3).y(1,2);
% ar = zeros(len,dim);
% for i = 1:dim,
%     for j = 1:len,
%         ar(j,i) = abs(PQ_3(j,i)-PQ_3_nes(j,i));
%     end
% end
% 
% sum(sum(ar))
% 
% Old_cost_2 = 100*abs(dtheta0--2.95)+sum(sum(ar))

%Optimization attempt 4
% abg = Dyn_Syst{1};
% Alpha = abg{1};
% Beta = abg{2};
% Gamma = abg{3};
% %  
% [l, s, m, I_m, I_r, N, g] = RobotParameters();
% % 
% PQ_3 = sol(3).CP;
% % 
% theta0_3 = sol(3).y(1,1);
% dtheta0_3 = sol(3).y(1,2)
% 
% A_3 = Alpha(l,m,s,I_r,PQ_3,theta0_3);
% B_3 = Beta(l,m,s,I_r,PQ_3,theta0_3);
% G_3 = Gamma(l,m,s,g,PQ_3,theta0_3);
% 
% ddtheta0_3 = (-B_3*dtheta0_3^2-G_3)/A_3
% % 
% PQ_4 = sol(4).CP;
% 
% theta0_4 = sol(4).y(1,1);
% dtheta0_4 = sol(4).y(1,2)
% 
% A_4 = Alpha(l,m,s,I_r,PQ_4,theta0_4);
% B_4 = Beta(l,m,s,I_r,PQ_4,theta0_4);
% G_4 = Gamma(l,m,s,g,PQ_4,theta0_4);
% 
% ddtheta0_4 = (-B_4*dtheta0_4^2-G_4)/A_4
% 
% Old_Cost = abs(ddtheta0_3-5)+abs(dtheta0--2)+abs(ddtheta0_4-10)
% 
% %Optimization Attempt 5
% 
% dtheta_end = sol(3).y(end,2);
% 
% Cost = ta*(abs(dtheta_end)-ddtheta0_4)

% temp = 0;

%Optimization Dazzle Ma nazzle
% for j = 1:2,
%     PQ_1 = sol(2*j).CP;
%     dq = [sol(2*j).y(end,2); getDPhi(PQ_1,sol(2*j).y(end,1))*sol(2*j).y(end,2)];  %dtheta2dphi_2dim(sol,PQ,i)'
%     [U1, U2] = getImpactModel(Dyn_Syst,sol,PQ_1,j*2);
%    
%     dq_ai = (U1\U2)*dq;
% 
%     %Switching parameters 
%     [theta0, phi0, dq_as] = switchParameters(sol(j*2).y(end,1), dq_ai,PQ_1,j*2);
%     dq_as
%     
%     if j == 1,
%         PQ_2 = sol(3).CP;
%         phi = theta2phi(sol(3).y(1,1),PQ_2);
%         dphi = getDPhi(PQ_2,sol(3).y(1,1));
%         ddphi = getDDPhi(PQ_2,sol(3).y(1,1));
%         dthe = sol(3).y(1,2);
%     else
%         PQ_2 = sol(1).CP;
%         phi = theta2phi(sol(1).y(1,1),PQ_2);
%         dphi = getDPhi(PQ_2,sol(1).y(1,1));
%         ddphi = getDDPhi(PQ_2,sol(1).y(1,1));
%         dthe = sol(1).y(1,2);
%     end
%     
%     y = zeros(4,1);
%     dy = zeros(4,1);
% 
%     for i = 1:4,
%         y(i) = phi0(i) - phi(i);
%         dy(i) = dq_as(i+1) - dphi(i)*dthe;
%     end
%     Y(:,j) = [y;dy];
% end
% 
% abs_Y = abs(Y)
% 
% gail = 100;
% snail = 10;
% costy_macCost = gail*sum(abs_Y(:,1))+snail*sum(abs_Y(:,2))
% pa = 0.001;
 
%nEW OPTIMIZATION SHEEET
    PQ_2 = sol(2).CP;
    PQ_3 = sol(3).CP;
    dq = [sol(2).y(end,2); getDPhi(PQ_2,sol(2).y(end,1))*sol(2).y(end,2)];  %dtheta2dphi_2dim(sol,PQ,i)'
    [U1, U2] = getImpactModel(Dyn_Syst,sol,PQ_2,2);
   
    dq_ai = (U1\U2)*dq
    
   [theta0_rand, phi0_rand, dq_as] = switchParameters(sol(2).y(end,1), dq_ai,PQ_2,2);
    
   dq_1 = dq_as(2:5)
   
   dq_2 = dPhi(PQ_3, sol(3).y(1,1), sol(3).y(1,2))'
    
    cost = zeros(1,4);
    for i = 1:4,
        cost(i) = dq_1(i) - dq_2(i);
    end
    
    cost(4) = cost(4);
    
    ErrorCost = sum(abs(cost));

    eagle = 1;
    
    
    
OPTIMIZATION_CRITERIA
switch lower(OPTIMIZATION_CRITERIA)
    case{'energy'}  
%         f = Old_Cost+temp;%+%abs(sum(energy_used)/(sum(m) * dx));
%         f =  abs(sum(energy_used)/(sum(m) * dx))*ypsil + temp + ga*abs(dtheta0-dtheta_end);  % Energy Optimization
%           f = ta*Old_cost_2+temp;  
%           f = Cost+temp
        
           f = ErrorCost*eagle + temp +  100*abs(dtheta0-dtheta_end);
%             f = costy_macCost + temp + 100*abs(dtheta0-dtheta_end);
            
    case{'speed'}   
        f = valeris*t_tot + temp + ga*abs(dtheta0-dtheta_end); % Speed Optimization
    otherwise
        f = temp + ga*abs(dtheta0-dtheta_end); % Optimization without specific contraints
end
                               
end

 