function [th, q, dq] = plotErrorCoordinates(LinSolExt)
%A function to plot the error of the generalized coordinates during the
%time evolution of all 4 steps of the gait.
%Input... 
%LinSolExt, the extended Linear Solution containing the error coordinates
%written in form of the generalized coordinates.
%Output...

n = length(LinSolExt{1}.y(:,1))+length(LinSolExt{2}.y(:,1))+length(LinSolExt{3}.y(:,1))+length(LinSolExt{4}.y(:,1));

th = zeros(n,2);
q = zeros(n,4);
dq = zeros(n,4);

%Extending the values of theta and dtheta for all 4 steps
for i = 1:2,
    th_temp = [];
    for j = 1:4,
        th_temp = [th_temp; LinSolExt{j}.y(:,i)];
    end
    th(:,i) = th_temp;
end

%Extending the values of q and dq for all 4 steps
for i = 1:4,
    q_temp = [];
    dq_temp = [];
    for j = 1:4,
        q_temp = [q_temp;LinSolExt{j}.q(:,i)];
        dq_temp = [dq_temp;LinSolExt{j}.dq(:,i)];
    end
    q(:,i) = q_temp;
    dq(:,i) = dq_temp;
end        

%Total time
t_tot = LinSolExt{1}.te+LinSolExt{2}.te+LinSolExt{3}.te+LinSolExt{4}.te;

t1 = LinSolExt{1}.t;
t2 = LinSolExt{2}.t+LinSolExt{1}.te;
t3 = LinSolExt{3}.t+LinSolExt{1}.te+LinSolExt{2}.te;
t4 = LinSolExt{4}.t+LinSolExt{1}.te+LinSolExt{2}.te+LinSolExt{3}.te;

t = [t1;t2;t3;t4];

%Plotting for theta and dtheta
figure,
hold on
plot(t,th(:,1),'r')
plot(t,th(:,2),'k')
title('Error Estimates of theta and dtheta')
xlabel('t')
ylabel('th, dth')
legend('th','dth','Location', 'SouthWest')
grid on
hold off


%Plotting for q
figure,
hold on
plot(t,q(:,1),'r')
plot(t,q(:,2),'k')
plot(t,q(:,3),'b')
plot(t,q(:,4),'g')
title('Error Estimates of q')
xlabel('t')
ylabel('q_1 q_2 q_3 q_4')
legend('q_1','q_2','q_3','q_4','Location', 'SouthWest')
grid on
hold off
    
%Plotting for dq
figure,
hold on
plot(t,dq(:,1),'r')
plot(t,dq(:,2),'k')
plot(t,dq(:,3),'b')
plot(t,dq(:,4),'g')
title('Error Estimates of dq')
xlabel('t')
ylabel('dq_1 dq_2 dq_3 dq_4')
legend('dq_1','dq_2','dq_3','dq_4','Location', 'SouthWest')
grid on
hold off     
        


