function [solExt, sol,dtheta_end] = simulateAllSteps(tspan, theta0, dtheta0,thetae_opt, P_tot, Dyn_Syst)
%Simulates all steps in a gait, to be returned to the optimization program.
%Thus update laws between each step are included.
%Input...
%tspan... time span for solution
%theta0... starting configuration of the ankle
%dtheta0... starting configuration of angular velocity of the ankle
%P_tot... Complete postional matrix
%Dyn_Syst... reference to the function handle of teh dynamic systems
%Output...
%SolExt... extended concatenated solution
%sol... struct containing the individual solutions for each step
%dthetae_end... ending value of dtheta in step 4 after impact and switch.
%Will later be checked towards dtheta0 to ensure they are equal.

global EPSILON
global NUM_VHC_PARAM
global Step_Length 

solExt = [];
solExt.error = true;
phi0 = P_tot(:,1,1)';
thetae = getThetae(Step_Length,NUM_VHC_PARAM-1);
thetae(1) = thetae_opt(1);
thetae(3) = thetae_opt(2);
P_prev = zeros(4,NUM_VHC_PARAM);
theta0_prev = 0;

for i = 1:4,
% for i = 1:3,
    %Updating Position matrix and smoothing the curves
    P = P_tot(:,:,i);
    
   
    if i>1,
        P = smoothPos(P,P_prev,theta0,thetae(i),theta0_prev,phi0,i);
    end
   
 
   PQ = Bezier(P,theta0,thetae(i),Dyn_Syst);
 
    [P_prev, theta0_prev] = updatePrevious(P,theta0,PQ,i);
      
    sol(i) = simulateSystem(tspan, theta0, dtheta0, Dyn_Syst,PQ,i);
    
 
    if even(i),
        
        %Impact model 
        dq = [sol(i).y(end,2); dPhi(PQ,sol(i).y(end,1),sol(i).y(end,2))'];  %dtheta2dphi_2dim(sol,PQ,i)'
        [U1, U2] = getImpactModel(Dyn_Syst,sol,PQ,i);
 
        dq_ai = (U1\U2)*dq;
           

        %Switching parameters 
        [theta0, phi0, dq_as] = switchParameters(sol(i).y(end,1), dq_ai,PQ,i);
        dtheta0 = dq_as(1);
        dtheta_end = dq_as(1);
 
    else
        theta0 = sol(i).y(end,1);
        dtheta0 = sol(i).y(end,2);
        phi0 = theta2phi(theta0,PQ);
    end
 
end

solExt = ExtendedSolution(sol,Dyn_Syst);
        