function [Lin_sol] = simulateLinearSystem(tspan,I0_y0_dy0,sol,PQ_tot,LinSys,Dyn_Syst,ite,K)
%Function to solve the Linear System created using transversal
%Linearization. It returns the Linear Solution which is considered to be
%valid near the optimal solution.
%Input...
%tspan... time span for each solution
%sol... solution of the non-linear system
%PQ... the coefficients of the parametrization of the DOF
%Linsys... function handles to the Linear System Function
%Dyn_Syst... Function Handles to the Dynamic Functions
%Output... Lin_sol, the solution to the linear system


%Function Handles and parameters
g_y = LinSys{1};
g_dy = LinSys{2};
g_ddy = LinSys{3};

abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

[l, s, m, I_m, I_r, N, g] = RobotParameters();
t_tot = sol(ite).te;
n = length(sol(ite).y(:,1));
n_reg = 101;

PQ = PQ_tot(:,:,ite);
K_set = [];
K_comp = cell(1,2);
ct = 0;
count = 0;

%Initial Parameters for the Transverse Coordinates
I0 = I0_y0_dy0(1);
y0 = I0_y0_dy0(2:5);
dy0 = I0_y0_dy0(6:9);

%Weight Matrices for the LQR Controller

% [Q,G,R,S,E] = getWeightMatrices(ite);

G = eye(9);         %G as in our literature
G(1,1) = 0.0000001;
% G(2,2) = 10;
% G(3,3) = 10;
% G(4,4) = 10;
% G(5,5) = 10;
% G(6,6) = 10;
% G(7,7) = 10;
% G(8,8) = 10;
% G(9,9) = 10;
Q = eye(9);         %Qmatrix, secondary weight for the transverse coord
R = eye(4);         %Gamma
S = zeros(9,4);     %g
E = eye(9);         %Big G


%Solving the First ODE

REFINE = 4;
 
options = odeset('Events',@Event,'Refine',REFINE,'RelTol',1e-3,'AbsTol',1e-6);
% doc odeset:
% Refine | Positive integer | Increases the number of output points by a factor of Refine.
% Events | Function handle  | Handle to a function that includes one or more
%                             event functions.

[ Lin_sol.t,Lin_sol.y,Lin_sol.te,Lin_sol.ye,Lin_sol.ie ] = ode45(@DynamicsEquations, tspan, [I0 y0(1) y0(2) y0(3) y0(4) dy0(1) dy0(2) dy0(3) dy0(4)], options);
% doc ode45:
% Corresponding entries in TE, YE,
% and IE return, respectively, the time at which an event
% occurs, the solution at the time of the event, and the index i of
% the event function that vanishes.

function [dx] = DynamicsEquations(t,x)
    count = count+1;
    min_t_distance = inf;
%   if t > sol(ite).t(end),
%       dx = zeros(9,1);
%       return
%   end
% t
    for i = 1:n,
        if t > sol(ite).t(end),
            rem = 0;
            ct_min = n;
            ct_pos = n;
        else,
            disc = abs(t-sol(ite).t(i));
            if disc < min_t_distance,
                if t >= sol(ite).t(i),
                    ct = i;
                    ct_min = ct;
                    ct_pos = ct+1;
                    if t == sol(ite).t(end),
                        rem = 1;
                        ct_pos = ct;
                    else,
                        rem = disc/(sol(ite).t(ct_pos)-sol(ite).t(ct_min));
                    end
                else,
                    ct = i-1;
                    ct_min = ct;
                    ct_pos = ct+1;
                    rem = 1- disc/(sol(ite).t(ct_pos)-sol(ite).t(ct_min));
                end
                min_t_distance = disc;
            end
        end
    end
%     rem
    
    
   
 
%      rem = mod(ct,1);
%   
%     ct_min = ct-rem;
%     ct_pos = ct_min+1;

%     ct_min = ct;
%     ct_pos = ct+1;
%     
%     if ct_min>n||ct_pos>n,
%         ct_min = n;
%         ct_pos= n;
%     end

    I_temp = x(1);
    y_temp = x(2:5);
    dy_temp = x(6:9);
    
%     the = (1-rem)*sol(ite).y(ct_min,1)+rem*sol(ite).y(ct_pos,1)
%     dthe = (1-rem)*sol(ite).y(ct_min,2)+rem*sol(ite).y(ct_pos,2);
%     t
    the = (1-rem)*sol(ite).y(ct_min,1)+rem*sol(ite).y(ct_pos,1);
    
    dthe = (1-rem)*sol(ite).y(ct_min,2)+rem*sol(ite).y(ct_pos,2);
    
    alpha = Alpha(l,m,s,I_r,PQ,the);
    
%     if alpha < 1e-5 && alpha > -1e-5,
%         alpha = sign(alpha)*1e-5;
%     end

    ddthe = -(Beta(l,m,s,I_r,PQ,the)*dthe^2+Gamma(l,m,s,g,PQ,the))/alpha;
    
    %Creating the Linear System
%     if dthe < 1e-5 && dthe > -1e-5,
%         gain = 1e-5;
%     else
        gain = (2*dthe)/(alpha);
%     end
 
%     the_gain = gain

%     if abs(gain) < 1e-9,
%         if gain >= 0,
%             gain = 1e-9;
%         else
%             gain = -1e-9;
%         end
%     end
   
 
    a_11 = -gain*Beta(l,m,s,I_r,PQ,the);
    a_12 = gain*g_y(PQ,l,m,s,the,g,dthe,ddthe)*(-1);
    a_13 = gain*g_dy(PQ,l,m,s,the,dthe)*(-1);
    b_1 = gain*g_ddy(PQ,l,m,s,the,I_r)*(-1);


     A = [a_11          a_12            a_13; %REMOVE
          zeros(4,1)    zeros(4,4)      eye(4);
          zeros(4,1)    zeros(4,4)      zeros(4,4)];

     B = [b_1;zeros(4,4); eye(4)];
     p = [-10 -9+2i -9-2i -8+4i -8-4i -7+6i -7-6i -6+8i -6-8i];
%      K = place(A,B,p);
%      x
     V = zeros(4,1);
     V(1) = -0.1; 
     
%         if t == 0,
%             A
%             B
%             x
%             A_new = (A-B*K)
%             A_new_rev = (A+B*K)
%             x_new = A_new*x
%             x_new_rev = A_new_rev*x
%             x_new_real = A*x
%         end
     
     
%     extracting K
    t_step = t/t_tot;
    pos = t_step*n_reg+1;
    rem = mod(pos,1);
    pos_1 = pos-rem;
    pos_2 = pos_1+1;
    
    if pos_1 > n_reg,
        pos_1 = n_reg;
    end
    if pos_2 > n_reg,
        pos_2 = n_reg;
    end
    
    K_reg = (1-rem)*K(:,:,pos_1) + rem*K(:,:,pos_2); 
     
     %Creating the Update Law
%      [X,L_m,K] = care(A,B,G,R,S,E);
    
      dx = (A+B*K_reg)*x;
%      K_set(:,:,count) = K;
     
%          dx = A*x;
%         v = zeros(4,1);
%         v(1) = -0.1;

%         dx = A*x;
end
Lin_sol.K = K_set;
function [value,isterminal,direction] = Event(t,x)
        value = t-t_tot;
        isterminal =1;
        direction = 0;
end
end





















