function [solExt, sol] = simulateSingleStep(tspan, theta0, dtheta0, PQ, Dyn_Syst,catchevent)
%Simulates a single step of the gait.
%Input...
%tspan.. time span for the step
%Theta0... starting angle for the ankle
%dtheta0... starting angular velocity for the ankle
%phi0 starting configuration for the joints
%P... coefficients for teh VHC
%Dyn_Syst... function handles to the dynamic equations.

REFINE = 4;

solExt = [];
solExt.error = true;

abg = Dyn_Syst{1};

if (nargin < 6)
    catchevent = true;
end

sol.CP = PQ;
 
if catchevent
    options = odeset('Events',@Event,'Refine',REFINE);
    % doc odeset:
    % Refine | Positive integer | Increases the number of output points by a factor of Refine.
    % Events | Function handle  | Handle to a function that includes one or more
    %                             event functions.
    
    [ sol.t,sol.y,sol.te,sol.ye,sol.ie ] = ode45(@DynamicsEquations, tspan, [theta0 dtheta0], options);
    % doc ode45:
    % Corresponding entries in TE, YE,
    % and IE return, respectively, the time at which an event
    % occurs, the solution at the time of the event, and the index i of
    % the event function that vanishes.

else
    options = odeset('Refine',REFINE);
    [ sol.t,sol.y ] = ode45(@DynamicsEquations, tspan, [theta0 dtheta0], options);
end


% check if the robot is above the floor, if the switch event occured and if
% the torque is feasible
[constraints_violated, torque] = mechanicalconstraints( sol, PQ, tspan, Dyn_Syst);

sol.torque = torque;

if constraints_violated
    sol.error = true;
else
    sol.error = false;
end
 
    function [dx] = DynamicsEquations(t, x)
        dx = ReducedDynamics(x, PQ, abg);
    end
 
    function [value,isterminal,direction] = Event(t,y)
        [value,isterminal,direction] = SwitchEvent(y);
    end

 
 if sol.error
        return;
 end

solExt.error = false; 
solExt = ExtendedSolution(sol);
end
