function [ sol ] = simulateSystem( tspan, theta0, dtheta0, Dyn_Syst , CP,ite, catchevent)
%SIMREDUCEDDYNAMICS simulates the reduced dynamics, which is
%defined by ReducedDynamics.m.
%
%[ sol ] = simReducedDynamics( tspan, theta0, dtheta0, CP, catchevent )
% tspan ... time span for simulation
% theta0 ... initial value, measure of progress
% dtheta0 ... initial value, derivative of measure of progress w.r.t. time
% CP ... constraint parameters
% catchevents ... if the SwitchEvent should be catched (default: true)
%returns:
% sol.t ... time values
% sol.y ... solution at the time values = [ theta(t), d/dt(theta(t)) ]
% sol.te ... time, at which the switch event occured
% sol.ye ... solution, at which the switch event occured
% sol.error ... true if one of the robot's joints is below the floor at a
%               any time, else false
% sol.torque ... torques in the joints

REFINE = 4;
count = 0;

abg = Dyn_Syst{1};
 
if (nargin < 7)
    catchevent = true;
end

sol.CP = CP;

if catchevent
    options = odeset('Events',@Event,'Refine',REFINE,'RelTol',1e-3,'AbsTol',1e-6);
    % doc odeset:
    % Refine | Positive integer | Increases the number of output points by a factor of Refine.
    % Events | Function handle  | Handle to a function that includes one or more
    %                             event functions.
    
    [ sol.t,sol.y,sol.te,sol.ye,sol.ie ] = ode45(@DynamicsEquations, tspan, [theta0 dtheta0], options);
    % doc ode45:
    % Corresponding entries in TE, YE,
    % and IE return, respectively, the time at which an event
    % occurs, the solution at the time of the event, and the index i of
    % the event function that vanishes.
   

else
    options = odeset('Refine',REFINE);
    [ sol.t,sol.y ] = ode45(@DynamicsEquations, tspan, [theta0 dtheta0], options);
end


% check if the robot is above the floor, if the switch event occured and if
% the torque is feasible

if count > 2000,
    sprintf('Out of Bounds')
end
[constraints_violated, torque, count] = mechanicalconstraints( sol, CP, tspan, Dyn_Syst, ite);
 
sol.torque = torque;

sol.count = count;

if constraints_violated
    sol.error = true;
else
    sol.error = false;
end
% 
% 
% 
    function [dx] = DynamicsEquations(t, x)
        count = count +1;
        dx = ReducedDynamics(x, CP, abg);
    end
% 
    function [value,isterminal,direction] = Event(t,y)
        [value,isterminal,direction] = SwitchEvent(y,ite,count);
    end
end