function [Trasol,Linsol,TrasolExt] = simulateTotalTransverseSystem(tspan,theta0,dtheta0,sol,PQ,Dyn_Syst,LinSys)
%Function designed to fully plan the evolution of the gait, utilizing the
%theory of Transverse Linearization to control to mimimize and thus create a
%global Stable system. A specific trajectory has been precalculated and the
%final goal will be to follow this trajectory as close as possible. 
%Input...
%tspan... total span of time per single evolution
%theta0... Initial starting configuration for theta
%dtheta0... Initial starting configuration for dtheta
%thetae_opt... optimized ending values for theta at certain intervals.
%sol... previously calculated solution.
%PQ... previously calculated and optimized values of the parametrization
%Dyn_Syst... function handles for the dynamic System
%LinSys... function handles for the 
%Output...
%TraSol... the solution of the transverse Linearization
%TraSolExt... the extended concanated solution

% %Storing the parameters
% TraSol = cell(4,1);
% LinSol = cell(4,1);

 %Initial Starting configurations
[q,dq,ddq,ddth] = getGeneralizedCoordinates(sol,PQ,Dyn_Syst,1);
q0 = q(1,:)
dq0 = dq(1,:)

%Running through the 4 simulatitions
% load('Constant_Gain');
% load('Linear_Solution_Origin_4_6');
% x_new = TrasolExt.x0;
% theta0 = x_new(1)
% dtheta0 = x_new(6)
% q0 = x_new(2:5)
% dq0 = x_new(7:10)
load('Regulator_2');
% load('Constant_Gain_7');
load('Linear_Transverse_Solution_2');


for i=1:4,
% for i=1:2,
% for i=1,
  
    % Simulating the Step
    [Trasol(i), Linsol(i)] = simulateTransverseSystem(tspan,theta0,dtheta0,q0,dq0,sol,PQ,Dyn_Syst,LinSys,Linsol_true,i,K{i});
    
    % Switching the step.
    %Updating the nonlinear solution
    if even(i),
        
        %Impact model 
%         dq_all = [TraSol(i).y(end,2); dPhi(PQ(:,:,i),TraSol(i).y(end,1),TraSol(i).y(end,2))'];  %dtheta2dphi_2dim(sol,PQ,i)'
        dq_all = Trasol(i).y(end,6:10)';
        [U1, U2] = getImpactModel(Dyn_Syst,Trasol,PQ(:,:,i),i);
        dq_ai = U1\U2*dq_all;
        
        %Switching parameters 
        [theta0, q0, dtheta] = switchParameters_10Dim(Trasol(i).y(end,1),Trasol(i).y(end,2:5),dq_ai,i);
          
        if i == 2,
            PQ_new = sol(3).CP;
            dq0 = dPhi(PQ_new, sol(3).y(1,1), sol(3).y(1,2))'       
        elseif i == 4,
            PQ_new = sol(1).CP;
            dq0 = dPhi(PQ_new, sol(1).y(1,1), sol(1).y(1,2))';
        end
            
 
        x_new = [theta0 q0 dtheta0 dq0'];
   else
        theta0 = Trasol(i).y(end,1)
        dtheta0 = Trasol(i).y(end,6)
        q0 = Trasol(i).y(end,2:5)
        dq0 = Trasol(i).y(end,7:10)
        x_new = [theta0 q0 dtheta0 dq0];
    end
    
    
end

%Finding the starting point for the transverse Coordinates for the next
%iteration of the gait

theta_opt_0 = sol(1).y(1,1);
dtheta_opt_0 = sol(1).y(1,2);
PQ = sol(1).CP;
x = zeros(10,1);
x(1) = theta0
x(2:5) = q0;
x(6) = dtheta0;
x(7:10) = dq0;
[l, s, m, I_m, I_r, N, g] = RobotParameters();
abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta_opt_0,the));

I_1 = x(6)^2 -exp(-2*integral(func1,theta_opt_0,x(1)))*dtheta_opt_0^2;
I_2 = exp(-2*integral(func1,theta_opt_0,x(1)))*integral(func2,theta_opt_0,x(1),'ArrayValued',true);
I = I_1+I_2;


            %The y parameters
phi = theta2phi(x(1),PQ);
dphi = getDPhi(PQ,x(1));
ddphi = getDDPhi(PQ,x(1));

y = zeros(4,1);
dy = zeros(4,1);

for k = 1:4,
    y(k) = x(k+1) - phi(k);
    dy(k) = x(k+6) - dphi(k)*x(6);
end

[I;y;dy];
Linsol(end).ye = [I;y;dy];

Trasol(end).yf = x';
            
TrasolExt = ExtendedSolutionTransverse(Trasol,Dyn_Syst,x_new);


end

    
    