function [Trasol,Linsol] = simulateTransverseSystem(tspan,theta0,dtheta0,q0,dq0,sol,PQ_tot,Dyn_Syst,LinSys,Linsol_true,ite,K)
%Function Designed to simulate a single step of the gait trying to mimic
%and already created "ideal" gait by way of transverse linearization. Thus
%if errors occur, ideally the control will manage to steer the system back
%towards the ideal solution.
%Input... 
%tspan... total span of time for completion of the solution
%theta0... starting value for theta0
%dtheta0... starting value for dtheta
%sol... Previously optimized solution, functioning as a reference solution.
%PQ... Optimal parameters for the parametrization of q2-q5 for the current
%step.
%Dyn_Syst... Function Handles to the Dynamic System.
%LinSys... function handles for the Linear System.
%ite... Current iteration 
%Output...
%TraSol... the complete solution of the transverse system stored.

REFINE = 4;
count = 0;

%Function Handles and parameters
g_y = LinSys{1};
g_dy = LinSys{2};
g_ddy = LinSys{3};

abg = Dyn_Syst{1};
Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};
tor = Dyn_Syst{2};

Syst_mat = Dyn_Syst{6};
M_syst = Syst_mat{1};
C_syst = Syst_mat{2}
G_syst = Syst_mat{3};

PCM = Dyn_Syst{3};
P_y = PCM{2,5};
n_reg = 101;

[l, s, m, I_m, I_r, N, g] = RobotParameters();
t_tot = sol(ite).te;
n = length(sol(ite).y(:,1));
t_tot = sol(ite).te;
PQ = PQ_tot(:,:,ite);
Trasol.CP = PQ;
Linsol.y = [];
Linsol.t = [];
Trasol.torque = [];
% Linsol = 1;
theta_opt_0 = sol(ite).y(1,1);
dtheta_opt_0 = sol(ite).y(1,2);
U_mat = zeros(1,4);
int_len = length(Linsol_true(ite).y(:,1));

% ct = 0;

%ERROR IMPLEMENTER FOR TESTING
if ite == 1,
    theta_start = theta0;%2.197941255349315;
    dtheta_start = dtheta0;%4.904797668758126;
%     q0(1) = q0(1)-0.1;
%     q0(2) = q0(2)+0.05;
%     q0(3) = q0(3)-0.12;
%     q0(4) = q0(4)+0.105;
%     dq0(1) = dq0(1)+0.05;
%     dq0(2) = dq0(2)+0.05;
%     dq0(3) = dq0(3)-0.2;
%     dq0(4) = dq0(4)-0.2;

else,
    theta_start = theta0;
    dtheta_start = dtheta0;
end

%Weight Matrices for the LQR Controller

% [Q,G,R,S,E] = getWeightMatrices(ite);

G = eye(9);         %G as in our literature
G(1,1) = 10;
G = G+100;
Q = eye(9);         %Qmatrix, secondary weight for the transverse coord
R = eye(4);         %Gamma
S = zeros(9,4);     %g
E = eye(9);         %Big G

%Using ODE45 to calculate the nonlinear System
 
options = odeset('Events',@Event,'Refine',REFINE,'RelTol',1e-3,'AbsTol',1e-6);
% doc odeset:
% Refine | Positive integer | Increases the number of output points by a factor of Refine.
% Events | Function handle  | Handle to a function that includes one or more
%                             event functions.

[ Trasol.t,Trasol.y,Trasol.te,Trasol.ye,Trasol.ie ] = ode45(@DynamicsEquations, tspan, [theta_start q0(1) q0(2) q0(3) q0(4) dtheta_start dq0(1) dq0(2) dq0(3) dq0(4)], options);
% doc ode45:
% Corresponding entries in TE, YE,
% and IE return, respectively, the time at which an event
% occurs, the solution at the time of the event, and the index i of
% the event function that vanishes.
   
    function [dx] = DynamicsEquations(t, x)
        count = count +1;
         
        %Extracting the Transverse Coordinates
%             x
%             t
 
                    
                func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
                func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta_opt_0,the));

                I_1 = x(6)^2 -exp(-2*integral(func1,theta_opt_0,x(1)))*dtheta_opt_0^2;
                I_2 = exp(-2*integral(func1,theta_opt_0,x(1)))*integral(func2,theta_opt_0,x(1),'ArrayValued',true);
                I = I_1+I_2;
 
%                 I = 0;
             

% %                 
%             %The y parameters
                phi = theta2phi(x(1),PQ);
                dphi = getDPhi(PQ,x(1));
                ddphi = getDDPhi(PQ,x(1));

                y = zeros(4,1);
                dy = zeros(4,1);

                for i = 1:4,
                    y(i) = x(i+1) - phi(i);
                    dy(i) = x(i+6) - dphi(i)*x(6);
                end
        

        
        %%     extracting K
        t_step = t/t_tot;
        pos = t_step*n_reg+1;
        rem = mod(pos,1);
        pos_1 = pos-rem;
        pos_2 = pos_1+1;

        if pos_1 > n_reg,
            pos_1 = n_reg;
        end
        if pos_2 > n_reg,
            pos_2 = n_reg;
        end
        if pos_1 < 1 || pos_2 < 1,
             pos_1 = 1;
             pos_2 = 1;
        end
        K_reg = (1-rem)*K(:,:,pos_1) + rem*K(:,:,pos_2); 

       
        %Calculate Controller Input V
            x_trans = [I;y;dy]
%               x_trans = zeros(9,1);
           


%             V = zeros(4,1);
              V = K_reg*x_trans

        
        %Calculate real output U from U = N(V-R)
            B_mat = [zeros(1,4); eye(4)];
        
            %The E-L matrices
                l1 = [1;dphi];
                l2 = [0;x(6)*ddphi];
                L_mat = [zeros(1,5); eye(4) zeros(4,1)] + [zeros(5,4) l1];
                dL_mat = [zeros(5,4) l2];

                E_mat = [eye(4) zeros(4,1)];

            %The N and R matrices
                N_mat = (E_mat/L_mat)*(M_syst(l,m,s,I_r,I_m,N,PQ,x(1:5))\B_mat);
                R_mat = (E_mat/L_mat)*(M_syst(l,m,s,I_r,I_m,N,PQ,x(1:5))\(-C_syst(l,m,s,PQ,x(1:5),x(6:10))*x(6:10)-G_syst(l,m,s,g,PQ,x(1:5)))-dL_mat*[x_trans(6:9); x(6)]);
            
            %Calculating System input u
                U= N_mat\(V-R_mat)
                
                for i=1:4,
                    if U(i) > 40;
                        U(i) = 40;
                    elseif U(i) < -40,
                        U(i) = -40;
                    end
                end
               
              
           ddq = M_syst(l,m,s,I_r,I_m,N,PQ,x(1:5))\(-C_syst(l,m,s,PQ,x(1:5),x(6:10))*x(6:10)- G_syst(l,m,s,g,PQ,x(1:5)) + B_mat*U);%U_exp);
          
        %Return the Solution
           dx = [x(6:10);ddq];
           
        

    end

    len = length(Trasol.y(:,1));
    for j=1:len,
%       j=len;
            x = Trasol.y(j,:)'
            t = Trasol.t(j);
            
            
            func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
            func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta_opt_0,the));

            I_1 = x(6)^2 -exp(-2*integral(func1,theta_opt_0,x(1)))*dtheta_opt_0^2;
            I_2 = exp(-2*integral(func1,theta_opt_0,x(1)))*integral(func2,theta_opt_0,x(1),'ArrayValued',true);
            I = I_1+I_2
%               I = 0;

  
%             %The y parameters
            phi = theta2phi(x(1),PQ);
            dphi = getDPhi(PQ,x(1));
            ddphi = getDDPhi(PQ,x(1));

            y = zeros(4,1);
            dy = zeros(4,1);

            for k = 1:4,
                y(k) = x(k+1) - phi(k);
                dy(k) = x(k+6) - dphi(k)*x(6);
            end

           x_trans = [I;y;dy];
           
            t_step = t/t_tot;
            pos = t_step*n_reg+1;
            rem = mod(pos,1);
            pos_1 = pos-rem;
            pos_2 = pos_1+1;

            if pos_1 > n_reg,
                pos_1 = n_reg;
            end
            if pos_2 > n_reg,
                pos_2 = n_reg;
            end
            if pos_1 < 1 || pos_2 < 1,
%                 break
            end
            K_reg = (1-rem)*K(:,:,pos_1) + rem*K(:,:,pos_2); 
           
           V = K_reg*x_trans;
           
           %Calculate real output U from U = N(V-R)
            B_mat = [zeros(1,4); eye(4)];
           
            %The E-L matrices
                l1 = [1;dphi];
                l2 = [0;x(6)*ddphi];
                L_mat = [zeros(1,5); eye(4) zeros(4,1)] + [zeros(5,4) l1];
                dL_mat = [zeros(5,4) l2];

                E_mat = [eye(4) zeros(4,1)];

            %The N and R matrices
                N_mat = (E_mat/L_mat)*(M_syst(l,m,s,I_r,I_m,N,PQ,x(1:5))\B_mat);
                R_mat = (E_mat/L_mat)*(M_syst(l,m,s,I_r,I_m,N,PQ,x(1:5))\(-C_syst(l,m,s,PQ,x(1:5),x(6:10))*x(6:10)-G_syst(l,m,s,g,PQ,x(1:5)))-dL_mat*[x_trans(6:9); x(6)]);
           
           U= N_mat\(V-R_mat);
           
           for i=1:4,
               if U(i) > 40;
                   U(i) = 40;
               elseif U(i) < -40,
                   U(i) = -40;
               end
           end
           
           Trasol.torque = [Trasol.torque; U'];     
           Linsol.y = [Linsol.y; x_trans'];
           Linsol.t = [Linsol.t; Trasol.t(j)];
    end
    Linsol.te = Trasol.te
    
     
    %Checking if the Solution is Valid
    [constraints_violated,count] = mechanicalconstraints10Dim( Trasol, tspan, Dyn_Syst, ite);

    Trasol.count = count;
    
    if constraints_violated
        Trasol.error = true;
    else
        Trasol.error = false;
    end

    function [value,isterminal,direction] = Event(t,x)
        [value,isterminal,direction] = SwitchEvent(x,ite,count,P_y);
    end
end

