%Function to store values of the function I(theta,dtheta,theta0,dtheta0) for
%a certain number of increments. This is because the integration process is
%both resource demanding and time consuming. Instead approximations will be
%made, and the value closest to a certain given value of theta will be
%instead found in a look-up table.

clear all
close all 
clc


%Load the Dynamic System Parameters
load('DynamicSystemParameters_3Dim');

abg = Dyn_Syst{1};

Alpha = abg{1};
Beta = abg{2};
Gamma = abg{3};

[l, s, m, I_m, I_r, N, g] = RobotParameters();

%Load specific solution
load('Solution_3_1_V2HD');
% PQ_tot = PQ;
size = 25+1;
Inc = zeros(4,2);

% Step 1
PQ = sol(1).CP;


theta0 = sol(1).y(1,1);
thetae = sol(1).y(end,1);
dtheta0 = sol(1).y(1,2);
dthetae = sol(1).y(end,2);

Inc(1,1) = (thetae-theta0)/size;
Inc(1,2) = (dthetae-dtheta0)/size;

Buf = 15;
A1 = zeros(size+2*Buf+1,size+2*Buf+1)+inf;

for i =1-Buf:size+Buf+1,
    theta = theta0 + (i-1)*Inc(1,1);
    if i <1,
        neg = Buf-abs(i)-1;
    else
        neg = Buf;
    end
    if i > size+1,
         pos = Buf-(i-size-1);
    else
        pos = Buf;
    end
%
    for j= i-neg:i+pos,
%     for j=1:size+1,
        dtheta = dtheta0 + (j-1)*Inc(1,2);
         
    
        %The Integration Function
        func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
        func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));

        I_1 = dtheta^2 -exp(-2*integral(func1,theta0,theta))*dtheta0^2;
        I_2 = exp(-2*integral(func1,theta0,theta))*integral(func2,theta0,theta,'ArrayValued',true);
        I = I_1+I_2;
        
        A1(i+Buf,j+Buf) = I
    end
end

% Step 2
PQ = sol(2).CP;

theta0 = sol(2).y(1,1);
thetae = sol(2).y(end,1);
dtheta0 = sol(2).y(1,2);
dthetae = sol(2).y(end,2);

Inc(2,1) = (thetae-theta0)/size;
Inc(2,2) = (dthetae-dtheta0)/size;

A2 = zeros(size+2*Buf+1,size+2*Buf+1)+inf;

for i =1-Buf:size+Buf+1,
    theta = theta0 + (i-1)*Inc(2,1);
    if i <1,
        neg = Buf-abs(i)-1;
    else
        neg = Buf;
    end
    if i > size+1,
         pos = Buf-(i-size-1);
    else
        pos = Buf;
    end
%
    for j= i-neg:i+pos,
%     for j=1:size+1,
        dtheta = dtheta0 + (j-1)*Inc(2,2);
         
    
        %The Integration Function
        func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
        func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));

        I_1 = dtheta^2 -exp(-2*integral(func1,theta0,theta))*dtheta0^2;
        I_2 = exp(-2*integral(func1,theta0,theta))*integral(func2,theta0,theta,'ArrayValued',true);
        I = I_1+I_2;
        
        A2(i+Buf,j+Buf) = I
    end
end

% Step 3
PQ = sol(3).CP;

theta0 = sol(3).y(1,1);
thetae = sol(3).y(end,1);
dtheta0 = sol(3).y(1,2);
dthetae = sol(3).y(end,2);

Inc(3,1) = (thetae-theta0)/size
Inc(3,2) = (dthetae-dtheta0)/size

A3 = zeros(size+2*Buf+1,size+2*Buf+1)+inf;


for i =1-Buf:size+Buf+1,
    theta = theta0 + (i-1)*Inc(3,1);
    if i <1,
        neg = Buf-abs(i)-1;
    else
        neg = Buf;
    end
    if i > size+1,
         pos = Buf-(i-size-1);
    else
        pos = Buf;
    end
%
    for j= i-neg:i+pos,
%     for j=1:size+1,
        dtheta = dtheta0 + (j-1)*Inc(3,2);
         
    
        %The Integration Function
        func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
        func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));

        I_1 = dtheta^2 -exp(-2*integral(func1,theta0,theta))*dtheta0^2;
        I_2 = exp(-2*integral(func1,theta0,theta))*integral(func2,theta0,theta,'ArrayValued',true);
        I = I_1+I_2;
        
        A3(i+Buf,j+Buf) = I
    end
end



% Step 4
PQ = sol(4).CP;

theta0 = sol(4).y(1,1);
thetae = sol(4).y(end,1);
dtheta0 = sol(4).y(1,2);
dthetae = sol(4).y(end,2);

Inc(4,1) = (thetae-theta0)/size;
Inc(4,2) = (dthetae-dtheta0)/size;

A4 = zeros(size+2*Buf+1,size+2*Buf+1)+inf;


for i =1-Buf:size+Buf+1,
    theta = theta0 + (i-1)*Inc(4,1);
    if i <1,
        neg = Buf-abs(i)-1;
    else
        neg = Buf;
    end
    if i > size+1,
         pos = Buf-(i-size-1);
    else
        pos = Buf;
    end
%
    for j= i-neg:i+pos,
%     for j=1:size+1,
        dtheta = dtheta0 + (j-1)*Inc(4,2);
         
    
        %The Integration Function
        func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
        func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));

        I_1 = dtheta^2 -exp(-2*integral(func1,theta0,theta))*dtheta0^2;
        I_2 = exp(-2*integral(func1,theta0,theta))*integral(func2,theta0,theta,'ArrayValued',true);
        I = I_1+I_2;
        
        A4(i+Buf,j+Buf) = I
    end
end


AI = cell(4,1);
AI{1} = A1;
AI{2} = A2;
AI{3} = A3;
AI{4} = A4;

dim = length(sol(1).CP(1,:));

PQ = zeros(4,dim,4);
PQ(:,:,1) = sol(1).CP;
PQ(:,:,2) = sol(2).CP;
PQ(:,:,3) = sol(3).CP;
PQ(:,:,4) = sol(4).CP;

Int_est = cell(3,1);
Int_est{1} = AI;
Int_est{2} = Inc;
Int_est{3} = Buf;

save('Solution_3_1_V2HD_INT','sol','solExt','Pos_e','PQ','dtheta_opt','thetae_opt','Int_est');






















% len = length(sol(1).y(:,1));
% Inte = zeros(len,1);
% load('A1');
% 
% for i = 1:len,
%     theta =  sol(1).y(i,1);%2.4118%theta0 + (10-1)*inc1_1;
%     dtheta = sol(1).y(i,2);%3.4194%dtheta0 + (10-1)*inc1_2
% 
% % func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
% %         func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));
% % 
% %         I_1 = dtheta^2 -exp(-2*integral(func1,theta0,theta))*dtheta0^2;
% %         I_2 = exp(-2*integral(func1,theta0,theta))*integral(func2,theta0,theta,'ArrayValued',true);
% %         I = I_1+I_2 
% % len = length(sol(4).y(:,1));
% % krak = zeros(len,1);
% % for k = 1:len,
% %     
% %     est_t1 = (sol(4).y(k,1)-sol(4).y(end,1))/inc1_1
% %     est_t2 = (sol(4).y(k,2)-sol(4).y(end,2))/inc1_2
% %     krak(k) = est_t1-est_t2;
% %     
% % end
% % % 

% load('Temp_Int');
% %     
% theta0 = sol(3).y(1,1);
% dtheta0 = sol(3).y(1,2);
% 
% PQ = sol(3).CP;
% 
% theta0 = sol(3).y(1,1);
% thetae = sol(3).y(end,1);
% dtheta0 = sol(3).y(1,2);
% dthetae = sol(3).y(end,2);
% 
% Inc(3,1) = (thetae-theta0)/size
% Inc(3,2) = (dthetae-dtheta0)/size
% 
% len =length(sol(3).y(:,1));
% K = zeros(len,2);
% S = zeros(len,1);
% InterMix  = zeros(len,2);
% for i = 1:len,
%     theta = sol(3).y(i,1);
%     dtheta = sol(3).y(i,2);
%     
%     t1 = Buf +(theta-theta0)/Inc(3,1);
%     t2 = Buf +(dtheta-dtheta0)/Inc(3,2);
%     K(i,1) = t1;
%     K(i,2) = t2;
%     S(i) = K(i,1)-K(i,2);
%     
%     rem1 = mod(t1,1);
%     rem2 = mod(t2,1);
%     P1 = (1-rem1);
%     P2 = rem1;
%     dP1 = (1-rem2);
%     dP2 = rem2;
% 
%     T1 = (t1-rem1)+1;
%     T2 = T1+1;
%     dT1 = (t2-rem2)+1;
%     dT2 = dT1+1;
%     if T1==12,
%         T2=12;
%     end
%     if dT1==12,
%         dT2=12;
%     end
%     
%     Inter = P1*dP1*A3(T1,dT1)+P1*dP2*A3(T1,dT2)+P2*dP1*A3(T2,dT1)+P2*dP2*A3(T2,dT2)
%     InterMix(i,1) = Inter;
%     
% %     The Integration Function
%         func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
%         func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));
% 
%         I_1 = dtheta^2 -exp(-2*integral(func1,theta0,theta))*dtheta0^2;
%         I_2 = exp(-2*integral(func1,theta0,theta))*integral(func2,theta0,theta,'ArrayValued',true);
%         I = I_1+I_2
%     InterMix(i,2) = I;
% end

% S













































%     rem1 = mod(t1,1)
%     rem2 = mod(t2,1)
%     P1 = (1-rem1);
%     P2 = rem1;
%     dP1 = (1-rem2);
%     dP2 = rem2;
% 
%     T1 = (t1-rem1)+1
%     T2 = T1+1
%     dT1 = (t2-rem2)+1
%     dT2 = dT1+1
%     if T1==12,
%         T2=12;
%     end
%     if dT1==12,
%         dT2=12;
%     end
%     Inter = P1*dP1*A1(T1,dT1)+P1*dP2*A1(T1,dT2)+P2*dP1*A1(T2,dT1)+P2*dP2*A1(T2,dT2)
%     Inte(i)= Inter;
% end
% 
% % 
% % theta1 = 2.2+T1*inc1_1;
% % theta2 = 2.2+T2*inc1_1;
% % dtheta1 = dtheta0+dT1*inc1_2;
% % dtheta2 = dtheta0+dT2*inc1_2;
% % 
% % func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
% %         func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));
% % 
% %         I_1 = dtheta1^2 -exp(-2*integral(func1,theta0,theta1))*dtheta0^2;
% %         I_2 = exp(-2*integral(func1,theta0,theta1))*integral(func2,theta0,theta1,'ArrayValued',true);
% %         I11 = I_1+I_2;
% %         
% % func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
% %         func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));
% % 
% %         I_1 = dtheta2^2 -exp(-2*integral(func1,theta0,theta1))*dtheta0^2;
% %         I_2 = exp(-2*integral(func1,theta0,theta1))*integral(func2,theta0,theta1,'ArrayValued',true);
% %         I12 = I_1+I_2 ;       
% %         
% % 
% % func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
% %         func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));
% % 
% %         I_1 = dtheta1^2 -exp(-2*integral(func1,theta0,theta2))*dtheta0^2;
% %         I_2 = exp(-2*integral(func1,theta0,theta2))*integral(func2,theta0,theta2,'ArrayValued',true);
% %         I21 = I_1+I_2;       
% % 
% % func1 = @(the) Beta(l,m,s,I_r,PQ,the)./Alpha(l,m,s,I_r,PQ,the);
% %         func2 = @(the) (2*Gamma(l,m,s,g,PQ,the)./Alpha(l,m,s,I_r,PQ,the))*exp(2*integral(func1,theta0,the));
% % 
% %         I_1 = dtheta2^2 -exp(-2*integral(func1,theta0,theta2))*dtheta0^2;
% %         I_2 = exp(-2*integral(func1,theta0,theta2))*integral(func2,theta0,theta2,'ArrayValued',true);
% %         I22 = I_1+I_2;
% % 
% %         
% % I_est = P1*dP1*I11+P1*dP2*I12+P2*dP1*I21+P2*dP2*I22        
% %         
% %         
% % % g = P1*dP1+P1*dP2+P2*dP1+P2*dP2      
% % end

        
        
        
        
        
        
