function [theta0, phi0, dtheta0] = switchParameters_10Dim(thetae,q_e,dq_ai,ite)
%Functions updating the starting configuration of the next step of the gait
%by switching the configuration of the semi-quad. This will occur after an
%impact, when the lifted leg hits the floor. At this instant theta will be
%switched to the new stance leg.
%Input...
%thetae... Ending value of previous theta
%dq_ai... New velocities of stance leg and joints after impact.
%Output... 
%theta0... new initial value of theta
%phi0... new initial configuration for the remainding joints
%dtehta0... new intial value for the angular velocity of theta
%ite... current iteration

F = [1 1 1 1 1;
     0 0 0 0 -1;
     0 0 0 -1 0;
     0 0 -1 0 0;
     0 -1 0 0 0];
r = [pi;0;0;0;0];

q_old = [thetae q_e];

if ite == 2,
    q_new = F*q_old'+r;
elseif ite == 4,
    q_new = F*q_old'-r;
end

dq_as = F*dq_ai;
theta0 = q_new(1);
dtheta0 = dq_as(1);
phi0 = q_new(2:5)';

end


