function [ phi ] = theta2phi(theta, PQ)
% THETA2PHI computes the value of the virtual holonomic constraints (phi) at
% concrete values of the measure of progress (theta).
%
%[ phi ] = theta2phi(theta, PQ)
% theta ... measure of progress, array
% PQ ... parameters of the polynomial virtual holonomic
%        constraint functions
%returns:
% phi ... values of the virtual holonomic constraint functions evaluated at
%         theta

n = size(PQ,2); % degree of vhc polynomial
phi = zeros(size(theta,1),4);
for i=1:length(theta)
    for j=1:4
        for k=1:n
            phi(i,j) = phi(i,j) + PQ(j,k)*theta(i)^(n-k);
        end
    end
end

phi = rem(phi,pi);

