function [Pos] = totalPos(x,N,Dim,thetae_opt,theta0,dtheta_opt,tspan,Dyn_Syst)
%Transform the optimized edited positions in array x, back into the
%complete Position matrix.

Pos = getPos(N,Dim);

%Normal optimization, no smoothing
% Pos(:,2,1) = x(1:4);
% Pos(:,3,1) = x(5:8);
% Pos(:,2,2) = x(9:12);
% Pos(:,2,3) = x(13:16);
% Pos(:,3,3) = x(17:20);
% Pos(:,2,4) = x(21:24);

%Optimization with smoothing
if Dim == 2,
    Pos(:,2,1) = x(1:4);
    Pos(:,3,1) = x(5:8);
    Pos(:,3,3) = x(9:12);
    
    
elseif Dim == 3,
    Pos(:,2,1) = x(1:4);
    Pos(:,3,1) = x(5:8);
    Pos(:,4,1) = x(9:12);
    Pos(:,3,2) = x(13:16);
    Pos(:,3,3) = x(17:20);
    Pos(:,4,3) = x(21:24);
    Pos(:,3,4) = x(25:28);
elseif Dim == 4,
    Pos(:,2,1) = x(1:4);
    Pos(:,3,1) = x(5:8);
    Pos(:,4,1) = x(9:12);
    Pos(:,5,1) = x(13:16);
    Pos(:,3,2) = x(17:20);
    Pos(:,4,2) = x(21:24);
    Pos(:,3,3) = x(25:28);
    Pos(:,4,3) = x(29:32);
    Pos(:,5,3) = x(33:36);
    Pos(:,3,4) = x(37:40);
    Pos(:,4,4) = x(41:44);
end

%Inserting for the symmetries of the positional matrices
thetae = getThetae(N,Dim);
thetae(1) = thetae_opt(1);
thetae(3) = thetae_opt(2);
dtheta0 = dtheta_opt;
for i = 1:4,
    %Updating Position matrix and smoothing the curves
    P = Pos(:,:,i);
    %P(:,1) = phi0';
    if i>1,
        P = smoothPos(P,P_prev,theta0,thetae(i),theta0_prev,phi0,i);
    end
    PQ = Bezier(P,theta0,thetae(i),Dyn_Syst);
    [P_prev, theta0_prev] = updatePrevious(P,theta0,PQ,i);
    %Solving the system     
    sol(i) = simulateSystem(tspan, theta0, dtheta0, Dyn_Syst,PQ,i);
 
    if even(i),
        
        %Impact model 
        dq = [sol(i).y(end,2); dPhi(PQ,sol(i).y(end,1),sol(i).y(end,2))'];
        [U1, U2] = getImpactModel(Dyn_Syst,sol,PQ,i);
        dq_ai = U1\U2*dq;
        
        %Switching parameters 
        [theta0, phi0, dq_as] = switchParameters(sol(i).y(end,1), dq_ai,PQ,i);
        dtheta0 = dq_as(1);
        dtheta_end = dq_as(1);
    else
        theta0 = sol(i).y(end,1);
        dtheta0 = sol(i).y(end,2);
        phi0 = theta2phi(theta0,PQ);
    end
    Pos(:,:,i) = P;
end

